
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/glew.h>
#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <QImage.h>
#include <qapplication.h>
#include <QDom.h>
#include <QFile.h>
#include <QdateTime.h>
#include <QDir.h>

#include <vector>

#include <vcg/Point3.h>
#include <vcg/Matrix44.h>

using namespace vcg;
using namespace std;

#include <CI/CISG.h>
#include <CI/CIMd2.h>
#include <MM/MMSG.h>

#include "../bmf/BMF_font.h"

QTime tm;
MMSG sg;
CIMd2 model;
BMF_Font *bfont;

bool animBall=false;

void Enter2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0, 640, 0, 480, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void Exit2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void DrawGLScene()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	float spec[4]={.4f,.4f,.4f,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	float lightpos[4]={0,0,1,0};
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
	glLightfv(GL_LIGHT0,GL_POSITION,lightpos);
	gluLookAt(2,2,4,0,0,0,0,1,0);
  glColor3f(1,.4f,.4f);
	glPushMatrix();
	glScalef(.04f,.04f,.04f);
	glRotatef(-180,0,1,0);
	glRotatef(-90,1,0,0);
	model.DrawTimedFrame(clock()/1000.0f);
  glPopMatrix();
	sg.drawGL(clock()/1000.0f);	
  
  Enter2D();
  	BMF_Begin(bfont);
		BMF_SetPos(20,20);
		BMF_Print("Current time is %i",clock());\
	  BMF_End();
  Exit2D();
	SDL_GL_SwapBuffers();
}

// Inizializzazione dello stato di OpenGL
void myGLInit()
{
  glewInit();
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);

	// abilita i texture environment necessari
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

  printf("loading font\n");
   bfont = BMF_Load("../bmf/data/fnt1.bmf");

 sg.root.initializeGL();

}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	myGLInit();
}


int main(int argc, char **argv) 
{  

  printf("Started \n");

  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);   
  
	/* Create a 640x480 OpenGL screen */
  if ( SDL_SetVideoMode(640, 480, 0, SDL_OPENGL |SDL_RESIZABLE) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

 /****  Creazione dello scenegraph*****/

 CISGGroup *mg = new CISGGroup();
 
 if(argc==1)
 {
 sg.XMLRead("../data/default.xml");
 sg.g->setTexture("../data/tile.png");
 }
 else
 {
   sg.XMLRead(argv[1]);
   sg.g->setTexture("../data/tile.png");
 }
	/* Loop, drawing and checking events */
	int done = 0;
	int ret;
  myGLReshapeFunc(640,480);  
	while ( ! done ) 
	{
    SDL_Event event;
	  ret=SDL_PollEvent(&event);
		if(ret)
			switch(event.type)
			{  
						case SDL_QUIT    : done = 1;  
															break ;
						case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
																				done = 1;
	

															break;
            case SDL_KEYUP : {
                              switch(event.key.keysym.sym )
                              {
                                case ' ' :
                                        printf("Started Animation");
                                        sg.g->setBallOnSelectionCenter();
                                        sg.g->generateFace();
                                        animBall=!animBall;
                                break;
                              }
                              break;
                             }
						case SDL_VIDEORESIZE	: //SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL | SDL_RESIZABLE);
																		myGLReshapeFunc(event.resize.w,event.resize.h); 
																		break;
			}  

		else 
    {
     if(animBall) sg.g->updateSpherePos(tm.elapsed()/1000.f);
     DrawGLScene();
    }
  }
  /* Cleaning up */
	// BMF_Free(bfont);
	SDL_Quit();
  return 1;
}
